/***********************************************************************
 *
 * PROJECT: PalmXQ
 * FILE:    setupform.c
 * AUTHOR:  Cheok Yan Cheng
 *
 * DESCRIPTION: Setup form for PalmXQ.
 *
 * LAST REVISION:   24/02/2002
 *
 ***********************************************************************/

#include <PalmOS.h>
#include "palmxqrsc.h"
#include "data.h"
#include "protos.h"
#include "defs.h"

static Boolean SetupOnMenu(UInt16 menuID);
static Boolean SetupOnPenDown(EventPtr event);
static void SetupListDrawFun(UInt16 itemNum, RectangleType *bounds,
                        Char **itemsText);
static Boolean SetupOnFrmOpen(EventPtr event);
static Boolean SetupOnLstSelect(EventPtr event);
static Boolean IsValidBoard(void);
static void SwapArray(UInt8 Array1[], UInt8 Array2[]);

static UInt8 SetupPiece[BRD_SIZE];
static UInt8 SetupColor[BRD_SIZE];

static Boolean SetupOnLstSelect(EventPtr event)
{
    Boolean handled = false;

    // This handled is always false so that the list drawing routine will
    // be handled by the system.
    return handled;
}

static Boolean SetupOnMenu(UInt16 menuID)
{
    Boolean    handled = false;

    switch (menuID)
    {
        case SetupDone:
            if(IsValidBoard())
            {
                switch(FrmAlert(SideAlert))
                {
                    // Light to move fisrt
                    case 0:
                        SwapArray(SetupColor, Color);
                        SwapArray(SetupPiece, Piece);

                        if(IsInCheck(DARK))
                        {
                            FrmCustomAlert(KingInCheckAlert, "Dark", NULL, NULL);
                            SwapArray(SetupColor, Color);
                            SwapArray(SetupPiece, Piece);
                        }
                        else
                        {
                            Side = LIGHT;
                            XSide = DARK;

                            FrmReturnToForm(0);
                            FrmUpdateForm(FrmGetFormId(FrmGetFormPtr(MainForm)),
                            frmRedrawnBoardCode);
                        }
                        break;

                    // Dark to move fisrt
                    case 1:
                        SwapArray(SetupColor, Color);
                        SwapArray(SetupPiece, Piece);

                        if(IsInCheck(LIGHT))
                        {
                            FrmCustomAlert(KingInCheckAlert, "Light", NULL, NULL);
                            SwapArray(SetupColor, Color);
                            SwapArray(SetupPiece, Piece);
                        }
                        else
                        {
                            Side = DARK;
                            XSide = LIGHT;

                            FrmReturnToForm(0);
                            FrmUpdateForm(FrmGetFormId(FrmGetFormPtr(MainForm)),
                            frmRedrawnBoardCode);
                        }
                        break;
                }
                    
                handled = true;
            }
            break;

        case SetupCancel:
            FrmReturnToForm(0);

            handled = true;
            break;
            
        default:
            break;
    }

    return handled;
}

static Boolean SetupOnPenDown(EventPtr event)
{
    Boolean handled = false;
    Int16 location;
    Int16 selection;
    FormType  *form;
    ListType  *list;

    form = FrmGetActiveForm();
    list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, SetupMoveList));

    if(ScrnToBrdCoord(form, SetupBoardGadget,
                    event->screenX, event->screenY, IsFlip, &location))
    {
        selection = LstGetSelection(list);

        SetupPiece[location] = (selection == 0)? EMPTY :
                               (selection - 1) % 7;
        SetupColor[location] = (selection == 0)? EMPTY :
                               (selection - 1) / 7;
        DrawBoard(form, SetupBoardGadget, SetupPiece, SetupColor, IsFlip);
        DrawPieceBound(form, SetupBoardGadget, location, IsFlip);

        handled = true;
    }

    return handled;
}

static Boolean SetupOnFrmOpen(EventPtr event)
{
    Boolean  handled = false;
    FormType  *form;
    ListType *list;

    form = FrmGetActiveForm();
    list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, SetupMoveList));

    FrmDrawForm(form);

    MemMove(SetupColor, Color, sizeof(SetupColor));
    MemMove(SetupPiece, Piece, sizeof(SetupPiece));

    DrawBoard(form, SetupBoardGadget, SetupPiece, SetupColor, IsFlip);

    LstSetDrawFunction(list, (ListDrawDataFuncPtr)SetupListDrawFun);
    LstSetListChoices(list, NULL, 15);
    LstDrawList(list);

    handled = true;

    return handled;
}

Boolean SetupHandleEvent(EventPtr event)
{
    Boolean  handled = false;
    FormType  *form;

    form = FrmGetActiveForm();

    switch (event->eType)
    {
        case frmOpenEvent:
            handled = SetupOnFrmOpen(event);
            break;

        case ctlSelectEvent:
            switch (event->data.ctlSelect.controlID)
            {
                default:
                    break;
            }
            break;

        case menuEvent:
            handled =
                SetupOnMenu(event->data.menu.itemID);
            break;

        case penDownEvent:
            handled =
                SetupOnPenDown(event);
            break;

        case frmUpdateEvent:
            break;

        default:
            break;
    }

    return handled;
}

static void SetupListDrawFun(UInt16 itemNum, RectangleType *bounds,
                        Char **itemsText)
{
    MemHandle bitmapH;
    BitmapType *bitmap;

    static Int16 bitmapID[15] = {
        NULL,
        BmpDarkPawn,
        BmpDarkAssistance,
        BmpDarkElephant,
        BmpDarkKnight,
        BmpDarkCannon,
        BmpDarkRook,
        BmpDarkKing,
        BmpLightPawn,
        BmpLightAssistance,
        BmpLightElephant,
        BmpLightKnight,
        BmpLightCannon,
        BmpLightRook,
        BmpLightKing };

    if(itemNum == 0)
    {
        WinDrawChars("---", 3,
                 bounds->topLeft.x,
                 bounds->topLeft.y);
    }
    else
    {
        bitmapH = DmGetResource(bitmapRsc, bitmapID[itemNum]);

        if(bitmapH)
        {
            bitmap = (BitmapPtr)MemHandleLock(bitmapH);
            MemHandleUnlock(bitmapH);

            // Draw now!
            //if(itemNum % 2 == 1)
            //{
                WinDrawBitmap(bitmap,
                            bounds->topLeft.x,
                            bounds->topLeft.y);
            //}
            //else
            //{
            //    WinDrawBitmap(bitmap,
            //        bounds->topLeft.x + bounds->extent.x - PIECEBMP_PIXEL - 1,
            //        bounds->topLeft.y + bounds->extent.y - PIECEBMP_PIXEL - 1);
            //
            //}
            DmReleaseResource(bitmapH);
        }

    }
}

static Boolean IsValidBoard()
{
    // Lookup table of king palace location (for DARK side)
    Int16 kingPalace[9] = {3, 4, 5, 12, 13, 14, 21, 22, 23};
    Int16 pieceNum[2][7] = {
        {5, 2, 2, 2, 2, 2, 1},
        {5, 2, 2, 2, 2, 2, 1} };

    Int16 i;
    Boolean kingFound;

    // Does both side kings exsit
    kingFound = false;
    for(i = 0; i < 9; i++)
    {        
        if(SetupPiece[kingPalace[i]] == KING &&
        SetupColor[kingPalace[i]] == DARK)
        {
            kingFound = true;
            break;
        }
    }

    if(!kingFound)
    {
        FrmCustomAlert(KingAlert, "Dark", NULL, NULL);
        return false;
    }

    kingFound = false;
    for(i = 0; i < 9; i++)
    {
        if(SetupPiece[89 - kingPalace[i]] == KING &&
        SetupColor[89 - kingPalace[i]] == LIGHT)
        {
            kingFound = true;
            break;
        }
    }

    if(!kingFound)
    {
        FrmCustomAlert(KingAlert, "Light", NULL, NULL);
        return false;
    }

    for(i = 0; i < BRD_SIZE; i++)
    {
        if(SetupPiece[i] ==EMPTY)
        {
            continue;
        }

        if((pieceNum[SetupColor[i]][SetupPiece[i]]--) <= 0)
        {
            FrmAlert(PieceNumAlert);
            return false;
        }

        if( (LegalPosition[SetupColor[i] == DARK?i:89-i] &
        MaskPiece[SetupPiece[i]]) == 0 )
        {
            FrmAlert(PiecePosAlert);
            return false;
        }
    }

    return true;
}

/***********************************************************************
 *
 * FUNCTION:    SwapArray
 *
 * DESCRIPTION: Swaps the contents of two arrays.
 *
 * PARAMETERS:  array1      Array to be swapped with array2.
 *              array2      Array to be swapped with array1.
 *
 *
 * RETURNED:    NONE
 *
 * COMMENT:     The size of array1 and array2 must == BRD_SIZE. Otherwise,
 *              the behaviour of this function is undetermined.
 *
 ***********************************************************************/
static void SwapArray(UInt8 Array1[], UInt8 Array2[])
{
    Int16 size;
    UInt8 tmp[BRD_SIZE];

    size = sizeof(UInt8) * BRD_SIZE;

    MemMove(tmp, Array2, size);
    MemMove(Array2, Array1, size);
    MemMove(Array1, tmp, size);
}
                                        
